"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var ast_1 = require("../utils/ast");
var ast_builders_1 = require("../utils/ast-builders");
var css_builders_1 = require("../utils/css-builders");
/**
 * Extracts styles from an expression.
 *
 * @param path Expression node
 */
var extractStyles = function (path) {
    if (t.isCallExpression(path.node) &&
        t.isIdentifier(path.node.callee) &&
        path.node.callee.name === 'css' &&
        t.isExpression(path.node.arguments[0])) {
        // css({}) call
        var styles = path.node.arguments[0];
        return styles;
    }
    if (t.isCallExpression(path.node) &&
        t.isMemberExpression(path.node.callee) &&
        t.isIdentifier(path.node.callee.property) &&
        path.node.callee.property.name === 'css' &&
        t.isExpression(path.node.arguments[0])) {
        // props.css({}) call
        var styles = path.node.arguments[0];
        return styles;
    }
    if (t.isTaggedTemplateExpression(path.node)) {
        var styles = path.node.quasi;
        return styles;
    }
    return undefined;
};
/**
 * Returns the children of a children as function expression.
 * Will throw if no children as function was found.
 *
 * @param path
 */
var getJsxChildrenAsFunction = function (path) {
    var children = path.node.children.find(function (node) { return t.isJSXExpressionContainer(node); });
    if (t.isJSXExpressionContainer(children) && t.isFunction(children.expression)) {
        return children.expression;
    }
    throw ast_1.buildCodeFrameError("ClassNames children should be a function\nE.g: <ClassNames>{props => <div />}</ClassNames>", path.node, path);
};
/**
 * Takes a class name component and transforms it into a compiled component.
 * This method will traverse the AST twice,
 * once to replace all calls to `css`,
 * and another to replace `style` usage.
 *
 * `<ClassNames>{}</ClassNames>`
 *
 * @param path Babel path - expects to be a JSX opening element.
 * @param state Babel state - should house options and meta data used during the transformation.
 */
exports.visitClassNamesPath = function (path, meta) {
    var _a;
    if (t.isJSXIdentifier(path.node.openingElement.name) &&
        path.node.openingElement.name.name !== ((_a = meta.state.compiledImports) === null || _a === void 0 ? void 0 : _a.ClassNames)) {
        // We aren't interested in this element. Bail out!
        return;
    }
    var variables = [];
    var sheets = [];
    // First pass to replace all usages of `css({})`
    path.traverse({
        Expression: function (path) {
            var styles = extractStyles(path);
            if (!styles) {
                // Nothing to do - skip.
                return;
            }
            var builtCss = css_builders_1.buildCss(styles, meta);
            var transformed = css_1.transformCss(builtCss.css);
            variables.push.apply(variables, builtCss.variables);
            sheets.push.apply(sheets, transformed.sheets);
            path.replaceWith(t.stringLiteral(transformed.classNames.join(' ')));
        },
    });
    // Second pass to replace all usages of `style`.
    path.traverse({
        Identifier: function (path) {
            if (path.node.name !== 'style' || path.parentPath.isProperty()) {
                // Nothing to do - skip.
                return;
            }
            var styleValue = variables.length
                ? t.objectExpression(ast_builders_1.buildCssVariablesProp(variables))
                : t.identifier('undefined');
            path.replaceWith(styleValue);
        },
    });
    // All done! Pick the children as function body and replace the original ClassNames node with it.
    var children = getJsxChildrenAsFunction(path);
    var body = ast_1.pickFunctionBody(children);
    path.replaceWith(ast_builders_1.compiledTemplate(body, sheets, meta));
};
//# sourceMappingURL=index.js.map